﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using InternshipSystem.Api.Queries;
using InternshipSystem.Api.Security;
using InternshipSystem.Core;
using InternshipSystem.Core.Entity;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Api.Controllers
{
    [ApiController]
    [Route("management/course")]
    public class CourseController : ControllerBase
    {
        private InternshipDbContext Context { get; }
        
        public CourseController(InternshipDbContext context)
        {
            Context = context;
        }

        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [Authorize(Policy = Policies.IsOverseer)]
        public async Task<ActionResult<IReadOnlyCollection<Course>>> GetCourses(CancellationToken cancellationToken) =>
            await Context.Courses
                .ToListAsync(cancellationToken);

        [HttpGet("{courseId}")]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [ProducesResponseType(StatusCodes.Status404NotFound)]
        [Authorize(Policy = Policies.IsOverseer)]
        public async Task<ActionResult<Course>> GetCourse(long courseId, CancellationToken cancellationToken)
        {
            var course = await Context.Courses
                .Where(c => c.Id == courseId)
                .FirstOrDefaultAsync(cancellationToken);

            if (course == null)
            {
                return NotFound();
            }

            return Ok(course);
        }
        
        [HttpPut]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status400BadRequest)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [ProducesResponseType(StatusCodes.Status404NotFound)]
        [Authorize(Policy = Policies.IsOverseer)]
        public async Task<ActionResult> UpsertCourse([FromBody] CourseForm courseForm, CancellationToken cancellationToken)
        {
            var validator = new CourseForm.Validator();
            var validationResult = await validator.ValidateAsync(courseForm, cancellationToken);

            if (!validationResult.IsValid)
            {
                return BadRequest(validationResult.ToString());
            }

            if (courseForm.Id.HasValue)
            {
                var course = await Context.Courses.FindAsync(courseForm.Id.Value);

                if (course == null)
                {
                    return NotFound();
                }

                course.UpdateCourse(courseForm.Name, courseForm.NameEng);
            }
            else
            {
                var newCourse = Course.CreateCourse(courseForm.Name, courseForm.NameEng);

                await Context.Courses.AddAsync(newCourse, cancellationToken);
            }

            await Context.SaveChangesAsync(cancellationToken);
            return Ok();
        }

        [HttpDelete("{courseId}")]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status400BadRequest)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [ProducesResponseType(StatusCodes.Status404NotFound)]
        [ProducesResponseType(StatusCodes.Status409Conflict)]
        [Authorize(Policy = Policies.IsOverseer)]
        public async Task<ActionResult> DeleteCourse(long courseId, CancellationToken cancellationToken)
        {
            var courseToDelete = await Context.Courses
                .Where(c => c.Id == courseId)
                .FirstOrDefaultAsync(cancellationToken);

            if (courseToDelete == null)
            {
                return NotFound();
            }

            var referencedEditions =
                await Context.Editions
                    .Include(e => e.Course)
                    .Where(e => e.Course.Id == courseToDelete.Id)
                    .CountAsync(cancellationToken);

            if (referencedEditions > 0)
            {
                return Conflict();
            }

            Context.Courses.Remove(courseToDelete);
            await Context.SaveChangesAsync(cancellationToken);
            return Ok();
        }
    }
}