﻿using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using InternshipSystem.Api.Queries;
using InternshipSystem.Api.Security;
using InternshipSystem.Core.Entity.Internship;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Api.Controllers
{
    [ApiController]
    [Route("internshipTypes")]
    public class InternshipTypesController : ControllerBase
    {
        private InternshipDbContext Context { get; }
        
        public InternshipTypesController(InternshipDbContext context)
        {
            Context = context;
        }
        
        /// <summary>
        /// Get internship types available for current edition
        /// </summary>
        /// <returns>List of internship types for current edition</returns>
        [HttpGet("current")]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [ProducesResponseType(StatusCodes.Status404NotFound)]
        [Authorize(Policy = Policies.RegisteredOnly)]
        public async Task<ActionResult<IEnumerable<InternshipType>>> GetInternshipTypesForEdition([FromServices] User user, CancellationToken cancellationToken)
        {
            var edition = 
                await Context.Editions
                    .Include(e => e.AvailableInternshipTypes)
                        .ThenInclude(e => e.InternshipType)
                    .Where(e => e.Id.Equals(user.EditionId))
                    .FirstOrDefaultAsync(cancellationToken: cancellationToken);

            if (edition == null)
            {
                return NotFound();
            }

            return Ok(edition.AvailableInternshipTypes.Select(e => e.InternshipType));
        }

        /// <summary>
        /// Get internship type by id
        /// </summary>
        /// <returns>Internship type</returns>
        /// <response code="200">Internship type returned successfully</response>
        /// <response code="401">This action is only available for authorized internship admin</response>
        /// <response code="404">Internship type not found</response>
        [HttpGet("{internshipTypeId}")]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [ProducesResponseType(StatusCodes.Status404NotFound)]
        [Authorize]
        public async Task<ActionResult<InternshipType>> GetInternshipType(long internshipTypeId, CancellationToken cancellationToken)
        {
            return await Context.InternshipTypes.FindAsync(internshipTypeId);
        }
        
        /// <summary>
        /// Get internship type list
        /// </summary>
        /// <returns>Internship type</returns>
        /// <response code="200">Internship type list returned successfully</response>
        /// <response code="401">This action is only available for authorized internship admin</response>
        [HttpGet]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [Authorize]
        public async Task<ActionResult<IEnumerable<InternshipType>>> SearchInternshipTypes([FromQuery] InternshipTypeSearchQuery searchQuery, CancellationToken cancellationToken)
        {
            return await Context.InternshipTypes
                .Where(t => string.IsNullOrEmpty(searchQuery.Label) || t.Label.Contains(searchQuery.Label))
                .Where(t => string.IsNullOrEmpty(searchQuery.LabelEng) || t.Label.Contains(searchQuery.LabelEng))
                .OrderBy(t => t.Label)
                .Skip(searchQuery.Page * searchQuery.PerPage)
                .Take(searchQuery.PerPage)
                .ToListAsync(cancellationToken);
        }

        /// <summary>
        /// Add or update internship type, only available for coordinator
        /// </summary>
        /// <response code="200">Internship type updated successfully</response>
        /// <response code="400">Invalid internship type form</response>
        /// <response code="401">This action is only available for authorized internship admin</response>
        /// <response code="404">Internship type with selected id do not exist</response>
        [HttpPut]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status400BadRequest)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [ProducesResponseType(StatusCodes.Status404NotFound)]
        [Authorize]
        public async Task<ActionResult> UpsertInternshipType([FromBody] InternshipTypeFrom internshipTypeFrom, CancellationToken cancellationToken)
        {
            var validator = new InternshipTypeFrom.Validator();
            var validationResult = await validator.ValidateAsync(internshipTypeFrom, cancellationToken);

            if (!validationResult.IsValid)
            {
                return BadRequest(validationResult.ToString());
            }

            if (internshipTypeFrom.Id.HasValue)
            {
                var internshipType = await Context.InternshipTypes.FindAsync(internshipTypeFrom.Id.Value);

                if (internshipType == null)
                {
                    return NotFound($"Internship type with id {internshipTypeFrom.Id} not found");
                }

                internshipType.Label = string.IsNullOrEmpty(internshipTypeFrom.Label) ? internshipType.Label : internshipTypeFrom.Label;
                internshipType.LabelEng = string.IsNullOrEmpty(internshipTypeFrom.LabelEng) ? internshipType.LabelEng : internshipTypeFrom.LabelEng;
                internshipType.Description = string.IsNullOrEmpty(internshipTypeFrom.Description) ? internshipType.Description : internshipTypeFrom.Description;
                internshipType.DescriptionEng = string.IsNullOrEmpty(internshipTypeFrom.DescriptionEng) ? internshipType.DescriptionEng : internshipTypeFrom.DescriptionEng;
            }
            else
            {
                var newInternshipType = new InternshipType
                {
                    Label = internshipTypeFrom.Label,
                    LabelEng = internshipTypeFrom.LabelEng,
                    Description = internshipTypeFrom.Description,
                    DescriptionEng = internshipTypeFrom.DescriptionEng,
                };

                await Context.InternshipTypes.AddAsync(newInternshipType, cancellationToken);
            }

            await Context.SaveChangesAsync(cancellationToken);
            return Ok();
        }

        /// <summary>
        /// Add or update internship type
        /// </summary>
        /// <response code="200">Internship type deleted successfully</response>
        /// <response code="401">This action is only available for authorized internship admin</response>
        /// <response code="404">Internship type with selected id do not exist</response>
        [HttpDelete("{internshipTypeId}")]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [ProducesResponseType(StatusCodes.Status404NotFound)]
        [Authorize]
        public async Task<ActionResult> DeleteInternshipType(long internshipTypeId, CancellationToken cancellationToken)
        {
            var internshipTypeToDelete = await Context.InternshipTypes
                .FirstOrDefaultAsync(t => t.Id.Equals(internshipTypeId), cancellationToken: cancellationToken);

            if (internshipTypeToDelete == null)
            {
                return NotFound($"Internship type with id: {internshipTypeId} does not exist");
            }

            Context.InternshipTypes.Remove(internshipTypeToDelete);
            await Context.SaveChangesAsync(cancellationToken);
            return Ok($"Internship type with id: {internshipTypeId} deleted successfully");
        }
    }
}