﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using AutoMapper;
using InternshipSystem.Api.Queries.SearchQuery;
using InternshipSystem.Api.Security;
using InternshipSystem.Core;
using InternshipSystem.Core.Entity;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Api.Controllers
{
    [Route("management/report")]
    public class ReportFieldsController : ControllerBase
    {
        private readonly InternshipDbContext _context;
        private readonly IMapper _mapper;

        public ReportFieldsController(InternshipDbContext context, IMapper mapper)
        {
            _context = context;
            _mapper = mapper;
        }

        [HttpGet("fields")]
        [Authorize(Policy = Policies.IsOverseer)]
        public async Task<IEnumerable<ReportField>> GetFields(FieldSearchQuery searchQuery, CancellationToken ct) =>
            await _context.ReportFields
                .Where(c => c.Label.ToLower().Contains(searchQuery.Label.ToLower()))
                .OrderBy(o => o.Label)
                .Skip(searchQuery.Page * searchQuery.PerPage)
                .Take(searchQuery.PerPage)
                .ToListAsync(ct);

        [HttpPost("fields")]
        [Authorize(Policy = Policies.IsOverseer)]
        public async Task<ActionResult> CreateField([FromBody] FieldCreateRequest request, CancellationToken ct)
        {
            ReportField field;

            switch (request.FieldType)
            {
                case FieldType.LongText:
                case FieldType.ShortText:
                    field = new ReportField(request.Label, request.LabelEng, request.Description, request.DescriptionEng, request.FieldType);
                    break;
                case FieldType.Select:
                case FieldType.Radial:
                case FieldType.Checkbox:
                    field = new ReportChoiceField(request.Label, request.LabelEng, request.Description, request.DescriptionEng, request.FieldType, request.Choices);
                    break;
                default:
                    return BadRequest("Unknown field type");
            }

            try
            {
                await _context.ReportFields.AddAsync(field, ct);
                await _context.SaveChangesAsync(ct);
            }
            catch (Exception e)
            {
                return BadRequest("Failed");
            }

            return Ok();
        }
    }

    public class FieldSearchQuery : SearchQuery
    {
        public string Label { get; set; } = "";
    }

    public class FieldCreateRequest
    {
        public long? Id { get; set; }
        public string Label { get; set; }
        public string LabelEng { get; set; }
        public string Description { get; set; }
        public string DescriptionEng { get; set; }
        public FieldType FieldType { get; set; }
        public string[] Choices { get; set; }
    }
}