﻿using System;
using System.Collections.Generic;
using System.Linq;
using InternshipSystem.Core.Entity;
using InternshipSystem.Core.Entity.Internship;
using InternshipSystem.Core.UglyOrmArtifacts;

namespace InternshipSystem.Core
{
    public class Edition
    {
        public Guid Id { get; set; }
        public DateTime EditionStart { get; set; }
        public DateTime EditionFinish { get; set; }
        public DateTime ReportingStart { get; set; }
        public Course Course { get; set; }
        public List<Internship> Internships { get; set; }
        public List<EditionSubject> AvailableSubjects { get; set; }
        public List<EditionInternshipType> AvailableInternshipTypes { get; set; }
        public List<ReportFieldEdition> ReportSchema { get; set; }

        public bool IsOpen => EditionFinish < DateTime.Today;

        public static Edition CreateEdition(DateTime start, DateTime end, DateTime reportingStart, Course course,
            IEnumerable<long> subjectsIds, IEnumerable<long> internshipTypesIds, IEnumerable<long> reportFieldIds)
        {
            var newEdition = CreateEdition(start, end, reportingStart, course);

            newEdition.AvailableSubjects =
                subjectsIds
                    .Select(s => new EditionSubject
                    {
                        Edition = newEdition,
                        InternshipSubjectId = s,
                    })
                    .ToList();

            newEdition.AvailableInternshipTypes =
                internshipTypesIds
                    .Select(i => new EditionInternshipType
                    {
                        Edition = newEdition,
                        InternshipTypeId = i,
                    })
                    .ToList();
            
            newEdition.ReportSchema =
                reportFieldIds
                    .Select(i => new ReportFieldEdition
                    {
                        Edition = newEdition,
                        ReportFieldId = i,
                    })
                    .ToList();

            return newEdition;
        }
            
        public static Edition CreateEdition(DateTime start, DateTime end, DateTime reportingStart, Course course)
        {
            return new Edition
            {
                EditionStart = start,
                EditionFinish = end,
                ReportingStart = reportingStart,
                Course = course,
                AvailableSubjects = new List<EditionSubject>(),
                AvailableInternshipTypes = new List<EditionInternshipType>(),
                ReportSchema = new List<ReportFieldEdition>()
            };
        }

        public void UpdateEdition(DateTime? start, DateTime? end, DateTime? reportingStart, Course course,
            IEnumerable<long> subjectsIds, IEnumerable<long> internshipTypesIds, IEnumerable<long> reportFieldIds)
        {
            EditionStart = start ?? EditionStart;
            EditionFinish = end ?? EditionFinish;
            ReportingStart = reportingStart ?? ReportingStart;
            Course = course;

            if (subjectsIds != null)
            {
                AvailableSubjects =
                    subjectsIds
                        .Select(s => new EditionSubject
                        {
                            InternshipSubjectId = s,
                        })
                        .ToList();
            }

            if (internshipTypesIds != null)
            {
                AvailableInternshipTypes =
                    internshipTypesIds
                        .Select(i => new EditionInternshipType
                        {
                            InternshipTypeId = i,
                        })
                        .ToList();
            }

            if (reportFieldIds != null)
            {
                ReportSchema =
                    reportFieldIds
                        .Select(i => new ReportFieldEdition
                        {
                            ReportFieldId = i,
                        })
                        .ToList();
            }
        }

        public void RegisterInternship(Student student)
        {
            if (Internships.Any(i => i.Student.Id == student.Id))
            {
                throw new ArgumentException("error.registration.already_registered");
            }
            
            var internship = Internship.CreateStudentsInternship(student);
            
            
            
            Internships.Add(internship);
        }

        public bool IsDateDuringEdition(DateTime start, DateTime end)
        {
            return start >= EditionStart && end <= EditionFinish;
        }

        public bool IsTypeAvailable(InternshipType internshipType)
        {
            return AvailableInternshipTypes.Any(e => e.InternshipType == internshipType);
        }

        public bool AreSubjectsAvailable(IEnumerable<long> internshipSubjects)
        {
            return internshipSubjects.All(s => AvailableSubjects.Any(su => su.InternshipSubjectId == s));
        }

        public bool IsValidDates => EditionStart <= EditionFinish;
    }
}